/*
* Copyright (c) 2006-2009 Erin Catto http://www.box2d.org
*
* This software is provided 'as-is', without any express or implied
* warranty.  In no event will the authors be held liable for any damages
* arising from the use of this software.
* Permission is granted to anyone to use this software for any purpose,
* including commercial applications, and to alter it and redistribute it
* freely, subject to the following restrictions:
* 1. The origin of this software must not be misrepresented; you must not
* claim that you wrote the original software. If you use this software
* in a product, an acknowledgment in the product documentation would be
* appreciated but is not required.
* 2. Altered source versions must be plainly marked as such, and must not be
* misrepresented as being the original software.
* 3. This notice may not be removed or altered from any source distribution.
*/

#ifndef SPHERE_STACK_H
#define SPHERE_STACK_H

class SphereStack : public Test
{
public:

	enum
	{
		e_count = 6
	};

	float ratio;
	float width;
	float height;
	float radius;

	int frameCount;	
	FILE *binFile;

	b2Body* ground;

	void CreateGround(bool createLine)
	{
		if (ground!=0)
		{
			m_world->DestroyBody(ground);
		}

		b2BodyDef bd;
		ground = m_world->CreateBody(&bd);

		float w = (width/2.0f);

		if ( createLine )
		{
			b2EdgeShape shape;
			shape.Set(b2Vec2(-w, 0.0f), b2Vec2(w, 0.0f));
			ground->CreateFixture(&shape, 0.0f);
		}

		b2EdgeShape shapeLeft;
		shapeLeft.Set(b2Vec2(-w, height), b2Vec2(-w, 0.0f));
		ground->CreateFixture(&shapeLeft, 0.0f);

		b2EdgeShape shapeRight;
		shapeRight.Set(b2Vec2(w, height), b2Vec2(w, 0.0f));
		ground->CreateFixture(&shapeRight, 0.0f);
	}

	SphereStack()
	{
		ratio = 0.1f;
		width = 384.0f * ratio;
		height = 205.0f * ratio;
		radius = 31.0f * ratio;

		frameCount = 0;
		binFile = fopen("..\\..\\..\\winApe\\src\\physics\\PhysicData.bin", "wb");

		ground = 0;
		CreateGround(true);

		{
			b2CircleShape shape;
			shape.m_radius = radius;

			for (int32 i = 0; i < e_count; ++i)
			{
				b2BodyDef bd;
				bd.type = b2_dynamicBody;
				bd.position.Set( 0.0, height + (radius) + (radius*i*4));

				m_bodies[i] = m_world->CreateBody(&bd);

				m_bodies[i]->CreateFixture(&shape, 0.01f);

				m_bodies[i]->SetLinearVelocity(b2Vec2(float(i-4)*0.25f, 1.1f));
			}
		}
	}

	void Step(Settings* settings)
	{
		Test::Step(settings);

		if ( frameCount == 65 )
		{
			CreateGround(false);
		}

		if ( frameCount > 76 )
		{
			if ( binFile != 0 )
			{
				fclose( binFile );
				exit(0);
				binFile = 0;
			}
		}
		else
		{
			for (int32 i = 0; i < e_count; ++i)
			{
				float x = ( (m_bodies[i]->GetWorldCenter().x*1.01f) + (width/2.0f) - radius );
				x /= ratio; // put back to normal ratio
				x /= 2.0f; // ratio 384->182
				float y = m_bodies[i]->GetWorldCenter().y + (radius*2.0f);
				y /= ratio; // put back to normal ratio
				y = (height/ratio) - y; // reverse Y

				y += ((radius*2)/ratio) * 1.5f; // make sure Y starts at top screen, discarding outside screen
				if ( y < 0.0f ) // clipping
				{
					y = -1.0f;
				}


				x += 1.0f;
				y += 5.0f;

				if ( y >= 205.0f+4.0f+48.0f )
				{
					y = 208.0f+48.0f;
				}

				unsigned char ucX = (unsigned char) (x);
				unsigned char ucY = (unsigned char) (y);

				unsigned short finalXY = (ucY * 256 ) + ( ucX );
				fwrite( &finalXY, 2, 1, binFile );
			}
		}
		frameCount++;

		//for (int32 i = 0; i < e_count; ++i)
		//{
		//	printf("%g ", m_bodies[i]->GetWorldCenter().y);
		//}

		//for (int32 i = 0; i < e_count; ++i)
		//{
		//	printf("%g ", m_bodies[i]->GetLinearVelocity().y);
		//}

		//printf("\n");
	}

	static Test* Create()
	{
		return new SphereStack;
	}

	b2Body* m_bodies[e_count];
};

#endif
